{$apptype console}

uses
  Math, SysUtils, Types;

var
  aa: array of array [1..4] of shortint;
  n, t: integer;

procedure Done(const Msg: string; Res: integer);
begin
  Writeln(Msg);
  Halt(Res);
end;


function Possible(A1, A2, A3, B1, B2, B3: shortint): boolean;
var
  aa, bb: array [1..4] of shortint;
  i, j: integer;
  d: shortint;
  ok: boolean;
begin
  Result := false;
  aa[1] := A1; aa[2] := A2; aa[3] := A3; aa[4] := n - (A1 + A2 + A3);
  bb[1] := B1; bb[2] := B2; bb[3] := B3; bb[4] := n - (B1 + B2 + B3);

  if (aa[4] < 0) or (bb[4] < 0) then
    Exit;

  // i -- - ,   1  2 ( --  2  1)
  for i := -n to n do begin
    d := i;
    ok := true;
    for j := 2 to 4 do begin
      if (aa[j] + d < 0) then begin
        ok := false;
        Break;
      end;
      if aa[j] < (aa[j] + d) - bb[j] then begin
        ok := false;
        Break;
      end;
      d := (aa[j] + d) - bb[j];
    end;
    if
      ok and (aa[1] - Max(i, 0) - Max(-d, 0) >= 0) and
      ((aa[1] - i + d) = bb[1])
    then begin
      Result := true;
      Exit;
    end;
  end;
end;


procedure CheckArmies(A, B, C, D: integer);
begin
  if (A < 0) or (B < 0) or (C < 0) or (D < 0) then
    Done(Format('Invalid army set: %d %d %d %d', [A, B, C, D]), 2);
  if (A + B + C + D <> n) then
    Done(Format('Too many armies used: %d %d %d %d', [A, B, C, D]), 1);
end;


procedure CheckDefense(AAIdx: integer; A, B, C, D: integer);
begin
  if
    (A < aa[AAIdx, 1]) or (B < aa[AAIdx, 2]) or
    (C < aa[AAIdx, 3]) or (D < aa[AAIdx, 4])
  then 
    Done(
      Format(
        'Defense failed in year %d: %d %d %d %d', [AAIdx + 1, A, B, C, D]), 1);
end;


var
  fi, fo, fa: textfile;
  i, j: integer;
  qo, qa: integer;
  a, b, c, d, pa, pb, pc, pd: integer;
begin
  AssignFile(fi, ParamStr(1));
  AssignFile(fo, ParamStr(2));
  AssignFile(fa, ParamStr(3));
  try
    {$I-}
    Reset(fi);
    Reset(fo);
    Reset(fa);
    if (IOResult <> 0) then
      Abort;
    {$I+}

    Read(fi, n, t);
    SetLength(aa, t);
    for i := 0 to t - 1 do
      for j := 1 to 4 do
        Read(fi, aa[i, j]);

    Read(fo, qo);
    Read(fa, qa);

    if qo > T then
      Done(Format('Q (%d) > T (%d)', [qo, t]), 1);

    if qo > 0 then begin
      Read(fo, pa, pb, pc, pd);
      CheckArmies(pa, pb, pc, pd);
      CheckDefense(0, pa, pb, pc, pd);
      for i := 2 to qo do begin
        Read(fo, a, b, c, d);
        CheckArmies(a, b, c, d);
        if not Possible(pa, pb, pc, a, b, c) then
          Done(
            Format(
              'Invalid army in year %d movement %d %d %d %d -> %d %d %d %d',
              [i, pa, pb, pc, pd, a, b, c, d]), 1);
        CheckDefense(i - 1, a, b, c, d);
        pa := a;
        pb := b;
        pc := c;
        pd := d;
      end;
    end;

    if qa > qo then
      Done(
        Format(
          'Defense was held during %d years, but %d is possible', [qo, qa]), 1)
    else if qa < qo then
      Done(
        Format(
          '!!! Answer (%d years) is worse than output (%d years)',
          [qa, qo]),
        3);
  except
    Halt(2);
  end;
  CloseFile(fi);
  CloseFile(fo);
  CloseFile(fa);
  Done('OK', 0);
end.
